/*
four-fours.cpp

・これはなに？
「4つの4」問題の一般化。
http://deztec.jp/x/05/faireal/faireal-5-27-index.html#d20527
http://deztec.jp/x/05/faireal/faireal-03-index.html#d20603a
このあたりのをわりとそのまま実装。ただし作者はC++についてほとんど素人なのでメモ
リのこととかよくわからない。GB単位でメモリを食うけどそれはアルゴリズム的に不可避
の事態なのかそれとも単にメモリまわりの手違いなのかは謎。あまり大きな領域を使うと
終了時にメモリが解放されなかったりするようだが不気味なだけで謎。

・あそびかた
作者が怠惰なせいでコマンドラインからパラメータを読むとかそういう器用な真似はして
いません。このすぐ下、#includeのちょっと下あたりと、main()関数の頭のほうにパラメ
ータがあると思うのでてきとうにいじってコンパイルしてください。MinGW g++4.5.2で動
作しました。TR1の使える環境だとよいかもなーとか、まあ細かい設定はコメント参照。

一応なんちゃってOpenMPに対応しているのでコンパイル時に-fopenmpフラグをつけておけ
ば並列化します。といってもあまりうまくない感じのような気はする。ループの順番替え
ればまたちがうかも。そのあたり対応するかも？でもmapはうまく並列化できそうもない
よなあ、とか。

あ、それと、出力されるTeXファイルはutf8なので、platex --kanji=utf8とするのがよい
と思われます。もっとも、英数字しか含まないので関係ないぽいですが。

・ライセンスについて
言い回しがよくわからない。NYSL http://www.kmonos.net/nysl/ を適用します。
まあ別にCC0でもなんでもいいのですが。とりあえずNYSLで。

	NYSL Version 0.9982
----------------------------------------

A. 本ソフトウェアは Everyone'sWare です。このソフトを手にした一人一人が、
   ご自分の作ったものを扱うのと同じように、自由に利用することが出来ます。

  A-1. フリーウェアです。作者からは使用料等を要求しません。
  A-2. 有料無料や媒体の如何を問わず、自由に転載・再配布できます。
  A-3. いかなる種類の 改変・他プログラムでの利用 を行っても構いません。
  A-4. 変更したものや部分的に使用したものは、あなたのものになります。
       公開する場合は、あなたの名前の下で行って下さい。

B. このソフトを利用することによって生じた損害等について、作者は
   責任を負わないものとします。各自の責任においてご利用下さい。

C. 著作者人格権は κねこせん(Twitter:@necocen) に帰属します。著作権は放棄します。

D. 以上の３項は、ソース・実行バイナリの双方に適用されます。

まあこんなのにライセンスもなにもなかろう。

パフォーマンス改善などについてなにかあれば
Twitter:@necocen
mailto:necocen@gmail.com
まで連絡いただけるとすこしうれしいです。

 */

#include <iostream>
#include <sstream>
#include <fstream>
#include <map>
#include <string>
#include <cmath>
#include <climits>
#include <vector>
#include <stack>

//なんちゃってOpenMP
//この変数はOpenMPを有効にすると自動で定義されるので触らなくてよい
#ifdef _OPENMP
#include "omp.h"
#endif

using namespace std;

//doubleの同値判定閾値
static const double EPS = 1e-9;

//これを越える値は探索しない
#define CUTOFF 1000000

//計算過程の非整数をすべてカット(小町算くらい探索空間が広いときに)
#define CUTOFF_NONINTEGER

//単項演算子の多重適用回数
//3にすると{4, 4, 4, 4}は無理っぽい
//2にして代わりに二重根号を導入した
#define UNARY_MAX 2

//数字の連結の可否
//(1,2,3)に対して12,23,123を認めるかどうか
#define ALLOW_COMBINE

//式の出力
//わりと出力が膨大になる(し、読めない)。デバッグ用
//#define OUTPUT_EXP

//進行状況の出力
//これも結構な量になるけど時間がかかってどこまで進んでるのか心配なときに。
#define OUTPUT_STEP

//TeX出力
//これを切ると普通に出力される
#define OUTPUT_TEX

//時間計測
#define MEASURE_TIME
#ifdef MEASURE_TIME
#include <time.h>
#endif

//tr1::unordered_mapを使う
//対応しているときは使った方が速い、はず
//#define USE_UNORDERED_MAP

//google-sparsehashを使う
//http://code.google.com/p/google-sparsehash/
//低速だがメモリをかなり節約できるので
//探索空間が広いときに使うとよい
//作者の環境では{4, 4, 4, 4, 4}が劇的に改善した(スワップが減ったせいであろう)
#define USE_SPARSE_HASH_MAP


//output switch
//コンソール出力用とTeX出力用でスイッチ
#ifdef OUTPUT_TEX
#define OS(X, Y) Y
#else
#define OS(X, Y) X
#endif

//括弧(通常用・TeX用)
#define PAREN_OPEN OS("(", "\\left(")
#define PAREN_CLOSE OS(")", "\\right)")

//内部データ型
//ほんとうはlong doubleのつもりでつけたldという名前だがlong doubleで持つのは重すぎるかなーと
typedef double ld;

//計算結果型
//firstがfalseならその結果は無効(ex.負の数の根号)
//さすがに複素数まで探索する気にはなれぬ:p
typedef pair<bool, ld> vV;

//単項演算子・二項演算子の関数ポインタ型
typedef vV (*unary_operator) (ld);
typedef vV (*binary_operator) (ld, ld);

typedef struct
{
	int arity;
	
	string rep_left;
	string rep_center;
	string rep_right;
	
	int precedence;
	
	bool associateLtoR;
	bool associateRtoL;

	bool needs_paren;
	bool needs_paren_left;
	bool needs_paren_right;

} operator_characteristics;

typedef struct
{
	char token;
	int cost;
	operator_characteristics op_chara;
	unary_operator op;
} unary;

unary make_unary(char token, string rep_left, string rep_right, int cost, int precedence, bool associateLtoR, bool associateRtoL, bool needs_paren, unary_operator op)
{
	unary u;
	u.token = token;
	u.cost = cost;
	
	u.op_chara.arity = 1;
	
	u.op_chara.rep_left = rep_left;
	u.op_chara.rep_right = rep_right;
	u.op_chara.rep_center = "";
	
	u.op_chara.precedence = precedence;
	u.op_chara.associateLtoR = associateLtoR;
	u.op_chara.associateRtoL = associateRtoL;
	
	u.op_chara.needs_paren = needs_paren;
	u.op_chara.needs_paren_left = u.op_chara.needs_paren_right = false;
	
	u.op = op;
	return u;
}

typedef struct
{
	char token;
	int cost;
	operator_characteristics op_chara;
	binary_operator op;
} binary;

binary make_binary(char token, string rep_left, string rep_center, string rep_right, int cost, int precedence, bool associateLtoR, bool associateRtoL, bool needs_paren_left, bool needs_paren_right, binary_operator op)
{
	binary b;
	b.token = token;
	b.cost = cost;
	
	b.op_chara.arity = 2;
	
	b.op_chara.rep_left = rep_left;
	b.op_chara.rep_center = rep_center;
	b.op_chara.rep_right = rep_right;
	
	b.op_chara.precedence = precedence;
	b.op_chara.associateLtoR = associateLtoR;
	b.op_chara.associateRtoL = associateRtoL;

	b.op_chara.needs_paren = false;
	b.op_chara.needs_paren_left = needs_paren_left;
	b.op_chara.needs_paren_right = needs_paren_right;
	
	b.op = op;
	return b;
}

typedef pair<int, string> knowledge_rep;

#ifdef USE_UNORDERED_MAP
#include <tr1/unordered_map>
using namespace tr1;
typedef unordered_map<ld, knowledge_rep> knowledge;
#else
#ifdef USE_SPARSE_HASH_MAP
#include <google/sparse_hash_map>
using google::sparse_hash_map;
using tr1::hash;
typedef sparse_hash_map<ld, knowledge_rep> knowledge;
#else
typedef map<ld, knowledge_rep> knowledge;
#endif
#endif

map<pair<int, int>, knowledge> knowledges;
vector<ld> seeds;
vector<unary> unaries;
vector<binary> binaries;


//トークン列から読める文字列へ
string represent(string tokens)
{
	stringstream ss;
	ss.clear();
	ss.str("");
	vector<ld>::iterator seed = seeds.begin();
	typedef pair<operator_characteristics, string> node;
	stack<node> opstack;
	operator_characteristics op_chara;
	for(string::iterator it = tokens.begin(); it != tokens.end(); it++)
	{
		char c = *it;
		if(c == '#')
		{
			ss << *(seed++);
			continue;
		}
		
		if (c == '@')
		{
			ss << *(seed++);
			operator_characteristics op_chara;
			op_chara.arity = 0;
			op_chara.precedence = 0;
			opstack.push(node(op_chara, ss.str()));
			ss.clear();
			ss.str("");
			continue;
		}
		
		bool found = false;

		//単項演算子？
		for(vector<unary>::iterator itu = unaries.begin(); itu != unaries.end(); itu++)
			if(c == (*itu).token)
			{
				found = true;
				op_chara = (*itu).op_chara;
				break;
			}
		if(found)
		{
			//unary op.
			operator_characteristics op_prev = opstack.top().first;
			string opr = opstack.top().second;
			opstack.pop();

			if(op_prev.precedence >= op_chara.precedence && op_chara.needs_paren)
				opr = PAREN_OPEN + opr + PAREN_CLOSE;
			opr = op_chara.rep_left + opr + op_chara.rep_right;
			opstack.push(node(op_chara, opr));
			continue;
		}

		//じゃあ、二項演算子？
		for(vector<binary>::iterator itb = binaries.begin(); itb != binaries.end(); itb++)
			if(c == (*itb).token)
			{
				found = true;
				op_chara = (*itb).op_chara;
				break;
			}
		if(found)
		{
			//binary op.
			operator_characteristics op_prev2 = opstack.top().first;
			string opr2 = opstack.top().second;
			opstack.pop();
			operator_characteristics op_prev1 = opstack.top().first;
			string opr1 = opstack.top().second;
			opstack.pop();

			if(op_chara.needs_paren_left && (op_prev1.precedence > op_chara.precedence || ((op_prev1.precedence == op_chara.precedence) && !(op_chara.associateLtoR))))
				opr1 = PAREN_OPEN + opr1 + PAREN_CLOSE;
			if(op_chara.needs_paren_right && (op_prev2.precedence > op_chara.precedence || ((op_prev2.precedence == op_chara.precedence) && !(op_chara.associateRtoL))))
				opr2 = PAREN_OPEN + opr2 + PAREN_CLOSE;
			string opr = op_chara.rep_left + opr1 + op_chara.rep_center + opr2 + op_chara.rep_right;
			opstack.push(node(op_chara, opr));
			continue;
		}
	}
	
	return opstack.top().second;
}

//整数っぽい？判定
//丸めてみて一緒だったら、とかそんなの
bool isInteger(ld x)
{
	if(fabs(x) > LLONG_MAX) return false;
	long long n = llround(x);
	return (fabs(x - n) <= EPS);
}

//数値っぽかったら数値にする
//数値ぽくなかったらそのまま返す
ld numerize(ld x)
{
	if(isInteger(x)) return llround(x);
	else return x;
}

//コスト判定
bool willReplace(int old_cost, int new_cost) {return (old_cost > new_cost);}

bool restriction_unary(ld x)
{
#ifdef CUTOFF_NONINTEGER
	return isInteger(x) && (fabs(x) <= CUTOFF) && (fabs(x) < EPS || (1.0 / fabs(x) < CUTOFF));
#else
	return (fabs(x) <= CUTOFF) && (fabs(x) < EPS || (1.0 / fabs(x) < CUTOFF));
#endif
}

bool restriction_binary(ld x)
{
#ifdef CUTOFF_NONINTEGER
	return isInteger(x) && (fabs(x) <= CUTOFF) && (fabs(x) < EPS || (1.0 / fabs(x) < CUTOFF));
#else
	return (fabs(x) <= CUTOFF) && (fabs(x) < EPS || (1.0 / fabs(x) < CUTOFF));
#endif
}

//nullaryでenrichする
//すなわち、演算子なしに数値として解釈されるものたち。初期値。
//knowledges[p]は存在しない
void enrich_nullary(pair<int, int> p)
{
	string st;
	for(int i = p.first; i < p.second; i++)
		st.append(1, '#');
	st.append(1, '@');
	ld val = 0;
	if(p.first == p.second)
		val = seeds.at(p.first);
	else
#ifdef ALLOW_COMBINE
		for(int i = p.first; i <= p.second; i++)
			val = val * 10 + seeds.at(i);
#else
		return;
#endif

#ifdef OUTPUT_EXP
	cout << "0-ary: " << val << endl;
#endif
	
	knowledge kl;
	if(fabs(val) <= CUTOFF)
		kl.insert(pair<ld, knowledge_rep>(val, knowledge_rep(0, st)));
	knowledges.insert(pair<pair<int, int>, knowledge>(p, kl));
	return;
}

//unaryでenrichする
//knowledges[p]の存在は保証されているとする(nullary, [binary])
void enrich_unary(pair<int, int> p)
{
#ifdef OUTPUT_STEP
	cout << "- I'm going to enrich unary [" << p.first << ", " << p.second << "]" << endl;
#endif
	for(int i = 0; i < UNARY_MAX; i++)
	{
		knowledge kl = knowledges[p];
#ifdef OUTPUT_STEP
		cout << "-- step " << (i + 1) << " in " << UNARY_MAX << "; for " << kl.size() << " knowledges." << endl;
#endif
		knowledge* kl2 = &(knowledges[p]);

		int unary_count = unaries.size();
#ifdef _OPENMP
#pragma omp parallel for
#endif
		for(int j = 0; j < unary_count; j++)
		{
#ifdef _OPENMP
			int thnum = omp_get_thread_num();
#endif
			unary un = unaries.at(j);
			unary_operator u = un.op;
#ifdef OUTPUT_STEP
#pragma omp critical
			{
				cout << "-- "
#ifdef _OPENMP
					<< "(Thread " << thnum << ") "
#endif
						<< " I'm going to consider \'" << un.token << "\'"<< endl;
			}
#endif
			for(knowledge::iterator it2 = kl.begin(); it2 != kl.end(); it2++)
			{
				ld val = (*it2).first;
				string rep = (*it2).second.second;
				string newrep = rep.append(1, un.token);
				int cost = (*it2).second.first;
				int newcost = cost + un.cost;
				vV res = u(val);
				ld newval = numerize(res.second);
				if(res.first && restriction_unary(newval))
				{
#pragma omp critical
					{
						if((*kl2).find(newval) == (*kl2).end() || willReplace((*((*kl2).find(newval))).second.first, newcost))
//						if((*kl2).find(newval) == (*kl2).end() || willReplace((*kl2)[newval].first, newcost))
						{
#ifdef OUTPUT_EXP
							cout << "--- "
#ifdef _OPENMP
								<< "(Thread " << thnum << ") "
#endif
									<< "1-ary: " << newrep << "=" << newval << " costs: " << newcost << endl;
#endif
							(*kl2)[newval] = knowledge_rep(newcost, newrep);
						}
					}
				}
			}
#ifdef OUTPUT_STEP
#pragma omp critical
			{
				cout << "-- "
#ifdef _OPENMP
					<< "(Thread " << thnum << ") "
#endif
						<< " I've considered \'" << un.token << "\'; there are " << knowledges[p].size() << " knowledges." << endl;
			}
#endif
		}
	}
#ifdef OUTPUT_STEP
		cout << "- I've enriched unary [" << p.first << ", " << p.second << "]" << endl;
#endif
	return;
}

//knowledges[p], knowledges[q]は保証
//knowledges[p ++ q (= r)]が存在(nullaryのみ)
void enrich_binary(pair<int, int> p, pair<int, int> q)
{
#ifdef OUTPUT_STEP
	cout << "- I'm going to combine [" << p.first << ", " << p.second << "], [" << q.first << ", " << q.second << "]." << endl;
#endif
	knowledge* kl1 = &(knowledges[p]);
	knowledge* kl2 = &(knowledges[q]);
	pair<int, int> r = pair<int, int>(p.first, q.second);
	knowledge* kl = &(knowledges[r]);
#ifdef OUTPUT_STEP
	cout << "- There are " << ((*kl1).size() * (*kl2).size()) << " knowledges." << endl;
#endif

	int binary_count = binaries.size();
#ifdef _OPENMP
#pragma omp parallel for
#endif
	for(int i = 0; i < binary_count; i++)
	{
#ifdef _OPENMP
		int thnum = omp_get_thread_num();
#endif
		binary bin = binaries.at(i);
		char op_token = bin.token;
		int op_cost = bin.cost;
		binary_operator b = bin.op;
#ifdef OUTPUT_STEP
#pragma omp critical
		{
			cout << "-- "
#ifdef _OPENMP
				<< "(Thread " << thnum << ") "
#endif
					<< "-- I'm going to consider \'" << op_token << "\'."<< endl;
		}
#endif
		for(knowledge::iterator it1 = (*kl1).begin(); it1 != (*kl1).end(); it1++)
		{
			for(knowledge::iterator it2 = (*kl2).begin(); it2 != (*kl2).end(); it2++)
			{
				ld val1 = (*it1).first;
				ld val2 = (*it2).first;
				string rep1 = (*it1).second.second;
				string rep2 = (*it2).second.second;
				int cost1 = (*it1).second.first;
				int cost2 = (*it2).second.first;
				string newrep = rep1 + rep2;
				newrep.append(1, op_token);
				int newcost = cost1 + op_cost + cost2;
				vV res = b(val1, val2);
				ld val = numerize(res.second);
				if(res.first && restriction_binary(val))
				{
#pragma omp critical
					{
						if((*kl).find(val) == (*kl).end() || willReplace((*((*kl).find(val))).second.first, newcost))
//						if((*kl).find(val) == (*kl).end() || willReplace((*kl)[val].first, newcost))
						{
#ifdef OUTPUT_EXP
							cout << "--- "
#ifdef _OPENMP
								<< "(Thread " << thnum << ") "
#endif
									<< "2-ary: " << newrep << "=" << val << " costs: " << newcost << endl;
#endif
							(*kl)[val] = knowledge_rep(newcost, newrep);
						}
					}
				}
			}
		}
#ifdef OUTPUT_STEP
#pragma omp critical
		{
			cout << "-- "
#ifdef _OPENMP
				<< "(Thread " << thnum << ") "
#endif
					<< "I've considered \'" << op_token << "\'; there are " << (*kl).size() << " knowledges."<< endl;
		}
#endif
	}
	knowledges.erase(p);
	knowledges.erase(q);
#ifdef OUTPUT_STEP
	cout << "- I've combined [" << p.first << ", " << p.second << "], [" << q.first << ", " << q.second << "]; there are " << knowledges[r].size() << " knowledges." << endl;
#endif
	return;
}

void think(pair<int, int> p)
{
	if(p.first > p.second) return;
	if(knowledges.find(p) != knowledges.end()) return;
#ifdef OUTPUT_STEP
	cout << "I'm going to think about [" << p.first << ", " << p.second << "]" << endl;
	cout << "- I'm going to enrich nullary [" << p.first << ", " << p.second << "]" << endl;
#endif
	enrich_nullary(p);
#ifdef OUTPUT_STEP
	cout << "- I've enriched nullary [" << p.first << ", " << p.second << "]" << endl;
#endif
	for(int i = p.first; i < p.second; i++)
	{
		think(pair<int, int>(p.first, i));
		think(pair<int, int>(i + 1, p.second));
		enrich_binary(pair<int, int>(p.first, i), pair<int, int>(i + 1, p.second));
	}
	enrich_unary(p);
#ifdef OUTPUT_STEP
	cout << "I've thought about [" << p.first << ", " << p.second << "]; there are " << knowledges[p].size() << " knowledges." << endl;
#endif
	return;
}

/* ----------------------------------------------------------------------------
	operators
	返す値はvV型。boolとldのpair。
	firstが計算結果が意味のあるものである(計算が正しく行われた)ことを示す
	secondは計算結果
	-------------------------------------------------------------------------- */
vV unary_inverse(ld x) {return vV(true, -x);}

vV unary_identity(ld x) {return vV(true, x);}

//根号
vV unary_sqrt(ld x)
{
	//四乗して整数でなければ弾く
	//あまり使いたくない仮定なので余裕があれば外したい
	//ただし{4,4,4,4}で0--1000の範囲では影響なし
	//{1,2,3,4}も影響なし
	if(!isInteger(pow(x, 4))) return vV(false, 0);
	if(x >= 0)
		return vV(true, sqrt(x));
	else if(fabs(x) < EPS)
		return vV(true, 0);
	else
		return vV(false, 0);
}

//二重根号
vV unary_sqrt2(ld x)
{
	//二乗して整数でなければ弾く
	//あまり使いたくない仮定なので余裕があれば外したい
	//ただし{4,4,4,4}で0--1000の範囲では影響なし
	//{1,2,3,4}も影響なし
	if(!isInteger(x * x)) return vV(false, 0);
	if(x >= 0)
	{
		if(sqrt(x) >= 0)
			return vV(true, sqrt(sqrt(x)));
		else
			return vV(false, 0);
	}
	else if(fabs(x) < (EPS * EPS))
			return vV(true, 0);
	else
		return vV(false, 0);
}

//三重根号(いらない)
vV unary_sqrt3(ld x)
{
	if(!isInteger(x)) return vV(false, 0);
	vV v = unary_sqrt2(x);
	if(v.first)
		return vV(true, sqrt(sqrt(sqrt(x))));
	else
		return vV(false, 0);
}

vV unary_fact(ld x)
{
	if(!isInteger(x)) return vV(false, 0);
	long long n = llround(x);
	if(n >= 0 && n <= 10) return vV(true, tgamma(n + 1));
	else return vV(false, n);
}

//総和記号(使いたくない)
vV unary_sigma(ld x)
{
	if(!isInteger(x)) return vV(false, 0);
	if(x <= 0) return vV(false, 0);
	long long n = numerize(x);
	return vV(true, n * (n + 1) / 2);
}

vV binary_plus(ld x, ld y) {return vV(true, x + y);}

vV binary_minus(ld x, ld y) {return vV(true, x - y);}

vV binary_product(ld x, ld y) {return vV(true, x * y);}

vV binary_divide(ld x, ld y)
{
	if(fabs(y) < EPS) return vV(false, 0);
	else return vV(true, x / y);
}

vV binary_power(ld x, ld y)
{
	if(!isInteger(y)) return vV(false, 0);
	if(fabs(x) < EPS && fabs(y) < EPS) return vV(true, 1);
	else return vV(true, pow(x, numerize(y)));
}

vV binary_root(ld x, ld y)
{
	if(y <= 0) return vV(false, 0);
	if(!isInteger(x)) return vV(false, 0);
	if(x <= 0) return vV(false, 0);
	if(fabs(x) < EPS) return vV(false, 0);
	return vV(true, pow(y, (1.0 / x)));
}

int main()
{

	/* --------------------------------------------------------------------------
	setting numbers
	このへんもっとなんとかならんのか
	-------------------------------------------------------------------------- */

//	long long int numbers[] = {1, 2, 3, 4, 5, 6, 7, 8, 9};
//	long long int numbers[] = {1, 2, 3, 4, 5;
	long long int numbers[] = {4, 4, 4, 4};
//	long long int numbers[] = {5, 5, 5, 5, 5};
	long long int startnum = 0;
	long long int endnum = 1000;

	//vectorにつっこむ
	int count = sizeof(numbers) / sizeof(numbers[0]);
	seeds.clear();
	for(int i = 0; i < count; i++)
		seeds.push_back(numbers[i]);

	
	/* --------------------------------------------------------------------------
	setting operators

	演算子を設定。優先度とかそういうのはてきとう。
	-------------------------------------------------------------------------- */

	//unaries
	//make_unary(char token, string rep_left, string rep_right, int cost, int precedence, bool associateLtoR, bool associateRtoL, bool needs_paren, unary_operator op)
	/*
┌────────┬───┬───┬─────┬───┐
│      単項演算子│ token│コスト│  優先順位│結合性│
├────────┼───┼───┼─────┼───┤
│ 単項マイナス(-)│     i│     2│         3│------│
├────────┼───┼───┼─────┼───┤
│        根号(√)│     s│     4│         1│------│
├────────┼───┼───┼─────┼───┤
│  二重根号(√√)│     q│     8│         1│------│
├────────┼───┼───┼─────┼───┤
│         階乗(!)│     !│     6│         2│------│
└────────┴───┴───┴─────┴───┘
		*/
	unaries.push_back(make_unary('i', "-", "", 2, 3, false, true, true, unary_inverse));
	unaries.push_back(make_unary('s', OS("sqrt", "\\sqrt{"), OS("", "}"), 4, 1, false, true, OS(true, false), unary_sqrt));
	unaries.push_back(make_unary('q', OS("sqrt(sqrt", "\\sqrt{\\sqrt{"), OS(")", "}}"), 8, 1, false, true, OS(true, false), unary_sqrt2));
	unaries.push_back(make_unary('!', "", "!", 6, 2, true, false, true, unary_fact));

	//binaries
	/*
┌────────┬───┬───┬─────┬───┐
│      二項演算子│ token│コスト│  優先順位│結合性│
├────────┼───┼───┼─────┼───┤
│         加算(+)│     +│     1│         6│結合的│
├────────┼───┼───┼─────┼───┤
│         減算(-)│     -│     2│         6│左→右│
├────────┼───┼───┼─────┼───┤
│         乗算(*)│     *│     3│         5│結合的│
├────────┼───┼───┼─────┼───┤
│         除算(/)│     /│     4│         5│左→右│
├────────┼───┼───┼─────┼───┤
│         冪乗(^)│     ^│     6│         3│左→右│
└────────┴───┴───┴─────┴───┘
	 */
	//make_binary(char token, string rep_left, string rep_center, string rep_right, int cost, int precedence, bool associateLtoR, bool associateRtoL, bool needs_paren_left, bool needs_paren_right, binary_operator op)
	binaries.push_back(make_binary('+', "", "+", "", 1, 6, true, true, true, true, binary_plus));
	binaries.push_back(make_binary('-', "", "-", "", 2, 6, true, false, true, true, binary_minus));
	binaries.push_back(make_binary('*', "", OS("*", "\\times "), "", 3, 5, true, true, true, true, binary_product));
	binaries.push_back(make_binary('/', OS("", "\\dfrac{"), OS("/", "}{"), OS("", "}"), 4, 5, true, false, OS(true, false), OS(true, false), binary_divide));
	binaries.push_back(make_binary('^', OS("", "{"), OS("^", "}^{"), OS("", "}"), 6, 3, true, false, true, OS(true, false), binary_power));


	/* --------------------------------------------------------------------------
	main routine
	-------------------------------------------------------------------------- */
	
#ifdef MEASURE_TIME
	clock_t start, end;
	double tm;
	start = clock();
#endif
	//main routine
	think(pair<int, int>(0, count - 1));
#ifdef MEASURE_TIME
	end = clock();
	tm = (double) (end - start) / CLOCKS_PER_SEC;
#endif

	
	/* --------------------------------------------------------------------------
	output
	-------------------------------------------------------------------------- */
	
#ifdef OUTPUT_TEX
	ofstream ofs;
	stringstream ss;
	ss.clear();
	ss.str("");
	for(int i = 0; i < count; i++)
		ss << numbers[i];
	ss << "_" << startnum << "-" << endnum;
	string texout = ss.str() + ".tex";
	ofs.open(texout.c_str(), ios::out);
	ofs << "\\documentclass[a4paper,fleqn]{jsarticle}" << endl;
	ofs << "\\usepackage{amsmath,amsfonts,amssymb,cases}" << endl;
	ofs << "\\allowdisplaybreaks[1]" << endl;
	ofs << "\\title{solution for generalized four-fours problem by numbers(";
	for(int i = 0; i < (count - 1); i++)
		ofs << numbers[i] << ", ";
	ofs << numbers[count - 1] << ") from " << startnum << " to " << endnum << ".}" << endl;
	ofs << "\\begin{document}" << endl;
	ofs << "\\maketitle" << endl;
	ofs << "\\begin{align*}" << endl;
#endif
	
	knowledge* kl = &(knowledges[pair<int, int>(0, count - 1)]);
	long long int c = 0;
	
	for(int i = startnum; i <= endnum; i++)
	{
		if((*kl).find(i) != (*kl).end())
		{
#ifdef OUTPUT_TEX
			ofs << i << "&=" << represent((*kl)[i].second) << "\\\\" << endl;
#else
			cout << i << " = " << represent((*kl)[i].second) << " costs: " << (*kl)[i].first << endl;
#endif
			c++;
		}
		else
		{
#ifdef OUTPUT_TEX
			ofs << i << "&: \\text{not found.}\\\\"  << endl;
#else
			cout << i << " : not found." << endl;
#endif
		}
	}
#ifdef OUTPUT_TEX
	ofs << "\\end{align*}" << endl;
	ofs << "Found " << c << " in " << (endnum - startnum + 1) << "."
#ifdef MEASURE_TIME
		<< " Elapsed " << tm << " seconds."
#endif
			<< endl;
	ofs << "\\end{document}" << endl;
	cout << "Found " << c << " in " << (endnum - startnum + 1) << "."
#ifdef MEASURE_TIME
		<< " Elapsed " << tm << " seconds."
#endif
			<< endl;
	cout << "output: " << texout << endl;
#else
	cout << "Found " << c << " in " << (endnum - startnum + 1) << "."
#ifdef MEASURE_TIME
		<< " Elapsed " << tm << " seconds."
#endif
			<< endl;
#endif

	return 0;
}
